
#!/bin/bash
# HAL NETFILTER FILTERING POLICY

# date : 08/2003
# author : jp
# mailto : jpgu@hal.hyperion

modprobe iptable_filter 1>&2 2> /dev/null
modprobe iptable_nat 1>&2 2> /dev/null
modprobe ip_nat_ftp 1>&2 2> /dev/null
modprobe ip_conntrack 1>&2 2> /dev/null
modprobe ip_conntrack_ftp 1>&2 2> /dev/null
modprobe iptable_mangle 1>&2 2> /dev/null
modprobe ipt_TOS 1>&2 2> /dev/null
modprobe ipt_MARK 1>&2 2> /dev/null
modprobe ipt_mark 1>&2 2> /dev/null
modprobe ipt_LOG 1>&2 2> /dev/null
modprobe ipt_MASQUERADE 1>&2 2> /dev/null
modprobe ipt_TCPMSS 1>&2 2> /dev/null
modprobe ipt_REJECT 1>&2 2> /dev/null
for TABLE in filter nat; do
iptables -t $TABLE -F 1>&2 2> /dev/null
iptables -t $TABLE -X 1>&2 2> /dev/null
done
for CHAIN in INPUT OUTPUT FORWARD; do
iptables -t filter -P $CHAIN ACCEPT 1>&2 2> /dev/null
done
enable () {
for file in $@; do
echo '1' > $file
done
}
disable () {
for file in $@; do
echo '0' > $file
done
}
	
		
# firewall default security policy
		
# We route the Internet Protocol !
enable /proc/sys/net/ipv4/ip_forward
# No source-routing !
disable /proc/sys/net/ipv4/conf/all/accept_source_route
# Ignore redirects from machines that are not listed as gateways
enable /proc/sys/net/ipv4/conf/all/secure_redirects
		
iptables -t filter -P INPUT DROP
iptables -t filter -A INPUT -i lo -j ACCEPT # loopback is opened
		
iptables -t filter -P OUTPUT DROP
iptables -t filter -A OUTPUT -o lo -j ACCEPT # loopback is opened
		
iptables -t filter -P FORWARD DROP
iptables -t filter -A FORWARD -o lo -j ACCEPT # loopback is opened
		
iptables -t filter -A INPUT -p TCP -m state --state NEW ! --syn -j DROP
iptables -t filter -A OUTPUT -p TCP -m state --state NEW ! --syn -j DROP
iptables -t filter -A FORWARD -p TCP -m state --state NEW ! --syn -j DROP
iptables -t filter -A INPUT -m state --state INVALID -j DROP
iptables -t filter -A OUTPUT -m state --state INVALID -j DROP
iptables -t filter -A FORWARD -m state --state INVALID -j DROP
disable /proc/sys/net/ipv4/conf/all/log_martians
		
iptables -t nat -i ppp0 -A PREROUTING -s 192.168.1.0/24 -j DROP
		
iptables -t filter -A INPUT -m state --state ESTABLISHED,RELATED -j ACCEPT
iptables -t filter -A OUTPUT -m state --state ESTABLISHED,RELATED -j ACCEPT
iptables -t filter -A FORWARD -m state --state ESTABLISHED,RELATED -j ACCEPT
echo '16535' > /proc/sys/net/ipv4/ip_conntrack_max
	
	
	
	
# input to host through ppp0
iptables -t filter -A INPUT -i ppp0 -m limit --limit 50/minute --limit-burst 50 -p icmp --icmp-type echo-request -j LOG --log-prefix "ICMP : " # log icmp from internet
# input to host through eth0
iptables -t filter -A INPUT -i eth0 -j ACCEPT # open
# output from host through ppp0
iptables -t filter -A OUTPUT -o ppp0 -j ACCEPT # open
iptables -t mangle -A POSTROUTING -o ppp0 -p tcp --dport 80 -j TOS --set-tos 16 # http
iptables -t mangle -A POSTROUTING -o ppp0 -p tcp --dport 20 -j TOS --set-tos 8 # ftp_data
# output from host through eth0
iptables -t filter -A OUTPUT -o eth0 -j ACCEPT # open
iptables -t mangle -A POSTROUTING -o eth0 -p tcp --dport 80 -j TOS --set-tos 16 # http
iptables -t mangle -A POSTROUTING -o eth0 -p tcp --dport 20 -j TOS --set-tos 8 # ftp_data		
	
	
# forward from LAN to INTERNET
iptables -t filter -N FORWARD-eth0-ppp0
iptables -t filter -A FORWARD -i eth0 -o ppp0 -j FORWARD-eth0-ppp0
iptables -t filter -A FORWARD-eth0-ppp0 -p tcp --dport 443 -j ACCEPT # https
iptables -t filter -A FORWARD-eth0-ppp0 -p tcp --dport 21 -j ACCEPT # ftp
iptables -t filter -A FORWARD-eth0-ppp0 -p tcp --dport 25 -j ACCEPT # smtp
iptables -t filter -A FORWARD-eth0-ppp0 -p tcp --dport 110 -j ACCEPT # pop3
iptables -t filter -A FORWARD-eth0-ppp0 -p tcp --dport 888 -j ACCEPT # cddb
iptables -t filter -A FORWARD-eth0-ppp0 -p udp --dport 53 -j ACCEPT # dns
iptables -t nat -A PREROUTING  -i eth0 -p tcp --dport 80 -j DNAT --to-destination 192.168.1.200:8080 # port forwarding for squid
iptables -t nat -A PREROUTING  -i eth0 -p tcp --dport 443 -j DNAT --to-destination 192.168.1.200:8080 # port forwarding for squid
iptables -t nat -A PREROUTING  -i eth0 -p tcp --dport 21 -j DNAT --to-destination 192.168.1.200:8080 # port forwarding for squid
iptables -t nat -A POSTROUTING  -o ppp0 -j MASQUERADE # dynamic nat
iptables -t mangle -A FORWARD -i eth0 -o ppp0 -p tcp --dport 80 -j TOS --set-tos 16 # http
iptables -t mangle -A FORWARD -i eth0 -o ppp0 -p tcp --dport 20 -j TOS --set-tos 8 # ftp_data
iptables -t filter -N FORWARD-ppp0-eth0
iptables -t filter -A FORWARD -i ppp0 -o eth0 -j FORWARD-ppp0-eth0
iptables -t filter -A FORWARD-ppp0-eth0 -j DROP # 	

iptables-save > /etc/sysconfig/iptables
iptables -nvL -t filter
iptables -nvL -t nat
iptables -nvL -t mangle
sysctl net.ipv4.ip_forward
sysctl net.ipv4.conf.all.accept_source_route
sysctl net.ipv4.conf.all.secure_redirects
sysctl net.ipv4.conf.all.log_martians
sysctl net.ipv4.ip_conntrack_max