#!/usr/bin/perl

# this file is part of the "xmlfilter" project. 
# Copyright (C) 2002 by Jean Philippe GUILLEMIN <jp.guillemin@free.fr>
# license: This software is under GPL version 2 of license
# date: 08 2004
# rev: 0.5

use XML::LibXSLT;
use XML::LibXML;

my $iptablesfile;
my $xslsheet = "/usr/local/xmlfilter/netfilter.xsl";

for ($I=0; $I < ($#ARGV+1) ; $I++) {
	if ($ARGV[$I] eq '-c') {
		$I++;
		$iptablesfile = $ARGV[$I];
	}
	if ($ARGV[$I] eq '-h') {
		&usage();
	}
}

if ($iptablesfile) {
	
	my $parser = XML::LibXML->new();
	my $xslt = XML::LibXSLT->new(); 

	
	open (XML, "$iptablesfile") || die "Error : can't open xml file !\n";
	undef $/;
	my $xmldoc=<XML>;
	
	#------------------------------------------------------------
	# xml transfomation 
	my $xml = $parser->parse_string($xmldoc);
	my $sheet = $parser->parse_string(<<'EOT');
<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
<xsl:output method="text" indent="no"/>
<xsl:strip-space elements="FILTER_RULE NAT_RULE LOG_RULE"/>
<!-- this XSLT sheet transforms a netfilter XML rule document into an iptables script -->

<xsl:template match="NETFILTER">
		<xsl:text>&#xa;#!/bin/bash</xsl:text>
		<xsl:text>&#xa;# </xsl:text><xsl:value-of select="@config_name"/>
		<xsl:text>&#xa;</xsl:text>
		<xsl:text>&#xa;# date : </xsl:text><xsl:value-of select="@date"/>
		<xsl:text>&#xa;# author : </xsl:text><xsl:value-of select="@author"/>
		<xsl:text>&#xa;# mailto : </xsl:text><xsl:value-of select="@email"/>
		<xsl:text>&#xa;</xsl:text>
		<xsl:text>&#xa;modprobe iptable_filter 1&#62;&#38;2 2&#62; /dev/null</xsl:text>
		<xsl:text>&#xa;modprobe iptable_nat 1&#62;&#38;2 2&#62; /dev/null</xsl:text>
		<xsl:text>&#xa;modprobe ip_nat_ftp 1&#62;&#38;2 2&#62; /dev/null</xsl:text>
		<xsl:text>&#xa;modprobe ip_conntrack 1&#62;&#38;2 2&#62; /dev/null</xsl:text>
		<xsl:text>&#xa;modprobe ip_conntrack_ftp 1&#62;&#38;2 2&#62; /dev/null</xsl:text>
		<xsl:text>&#xa;modprobe iptable_mangle 1&#62;&#38;2 2&#62; /dev/null</xsl:text>
		<xsl:text>&#xa;modprobe ipt_TOS 1&#62;&#38;2 2&#62; /dev/null</xsl:text>
		<xsl:text>&#xa;modprobe ipt_MARK 1&#62;&#38;2 2&#62; /dev/null</xsl:text>
		<xsl:text>&#xa;modprobe ipt_mark 1&#62;&#38;2 2&#62; /dev/null</xsl:text>
		<xsl:text>&#xa;modprobe ipt_LOG 1&#62;&#38;2 2&#62; /dev/null</xsl:text>
		<xsl:text>&#xa;modprobe ipt_MASQUERADE 1&#62;&#38;2 2&#62; /dev/null</xsl:text>
		<xsl:text>&#xa;modprobe ipt_TCPMSS 1&#62;&#38;2 2&#62; /dev/null</xsl:text>
		<xsl:text>&#xa;modprobe ipt_REJECT 1&#62;&#38;2 2&#62; /dev/null</xsl:text>
		<xsl:text>&#xa;for TABLE in filter nat; do</xsl:text>
		<xsl:text>&#xa;iptables -t $TABLE -F 1&#62;&#38;2 2&#62; /dev/null</xsl:text>
		<xsl:text>&#xa;iptables -t $TABLE -X 1&#62;&#38;2 2&#62; /dev/null</xsl:text>
		<xsl:text>&#xa;done</xsl:text>
		<xsl:text>&#xa;for CHAIN in INPUT OUTPUT FORWARD; do</xsl:text>
		<xsl:text>&#xa;iptables -t filter -P $CHAIN ACCEPT 1&#62;&#38;2 2&#62; /dev/null</xsl:text>
		<xsl:text>&#xa;done</xsl:text>
		<xsl:text>&#xa;enable () {</xsl:text>
		<xsl:text>&#xa;for file in $@; do</xsl:text>
		<xsl:text>&#xa;echo '1' > $file</xsl:text>
		<xsl:text>&#xa;done</xsl:text>
		<xsl:text>&#xa;}</xsl:text>
		<xsl:text>&#xa;disable () {</xsl:text>
		<xsl:text>&#xa;for file in $@; do</xsl:text>
		<xsl:text>&#xa;echo '0' > $file</xsl:text>
		<xsl:text>&#xa;done</xsl:text>
		<xsl:text>&#xa;}</xsl:text>
		<xsl:apply-templates/>
		<xsl:text>&#xa;iptables-save > /etc/sysconfig/iptables</xsl:text>
		<xsl:text>&#xa;iptables -nvL -t filter</xsl:text>
		<xsl:text>&#xa;iptables -nvL -t nat</xsl:text>
		<xsl:text>&#xa;iptables -nvL -t mangle</xsl:text>
		<xsl:text>&#xa;sysctl net.ipv4.ip_forward</xsl:text>
		<xsl:text>&#xa;sysctl net.ipv4.conf.all.accept_source_route</xsl:text>
		<xsl:text>&#xa;sysctl net.ipv4.conf.all.secure_redirects</xsl:text>
		<xsl:text>&#xa;sysctl net.ipv4.conf.all.log_martians</xsl:text>
		<xsl:text>&#xa;sysctl net.ipv4.ip_conntrack_max</xsl:text>
	</xsl:template>
	
	<xsl:template match="POLICY">
		<xsl:apply-templates/>
	</xsl:template>		
	
	<xsl:template match="POLICY/INPUT_POLICY">
		<xsl:text>&#xa;iptables -t filter -P INPUT </xsl:text><xsl:value-of select="@policy"/>
		<xsl:text>&#xa;iptables -t filter -A INPUT -i lo -j ACCEPT # loopback is opened</xsl:text>
		
	</xsl:template>
	
	<xsl:template match="POLICY/OUTPUT_POLICY">
		<xsl:text>&#xa;iptables -t filter -P OUTPUT </xsl:text><xsl:value-of select="@policy"/>
		<xsl:text>&#xa;iptables -t filter -A OUTPUT -o lo -j ACCEPT # loopback is opened</xsl:text>
		
	</xsl:template>
	
	<xsl:template match="POLICY/FORWARD_POLICY">
		<xsl:text>&#xa;iptables -t filter -P FORWARD </xsl:text><xsl:value-of select="@policy"/>
		<xsl:text>&#xa;iptables -t filter -A FORWARD -o lo -j ACCEPT # loopback is opened</xsl:text>
	</xsl:template>
	
	<xsl:template match="POLICY/ROUTING">
		<xsl:if test="@ip_forwarding='yes'">
			<xsl:text>&#xa;# We route the Internet Protocol !</xsl:text>
			<xsl:text>&#xa;enable /proc/sys/net/ipv4/ip_forward</xsl:text>
		</xsl:if>
		<xsl:if test="@ip_forwarding='no'">
			<xsl:text>&#xa;disable /proc/sys/net/ipv4/ip_forward</xsl:text>
		</xsl:if>
		<xsl:if test="@no_source_routing='yes'">
			<xsl:text>&#xa;# No source-routing !</xsl:text>
			<xsl:text>&#xa;disable /proc/sys/net/ipv4/conf/all/accept_source_route</xsl:text>
		</xsl:if>
		<xsl:if test="@no_source_routing='no'">
			<xsl:text>&#xa;enable /proc/sys/net/ipv4/conf/all/accept_source_route</xsl:text>
		</xsl:if>
		<xsl:if test="@secure_redirects='yes'">
			<xsl:text>&#xa;# Ignore redirects from machines that are not listed as gateways</xsl:text>
			<xsl:text>&#xa;enable /proc/sys/net/ipv4/conf/all/secure_redirects</xsl:text>
		</xsl:if>
		<xsl:if test="@secure_redirects='no'">
			<xsl:text>&#xa;disable /proc/sys/net/ipv4/conf/all/secure_redirects</xsl:text>
		</xsl:if>
	</xsl:template>
	
	<xsl:template match="POLICY/ANTISPOOF">
		<xsl:if test="@homenet!='none'">
			<xsl:text>&#xa;iptables -t nat -i </xsl:text><xsl:value-of select="@extern_if"/>
			<xsl:text> -A PREROUTING -s </xsl:text><xsl:value-of select="@homenet"/><xsl:text> -j DROP</xsl:text>
		</xsl:if>
	</xsl:template>
	
	<xsl:template match="POLICY/STATEFULL">
		<xsl:if test="@statefull_inspection='yes'">
			<xsl:text>&#xa;iptables -t filter -A INPUT -m state --state ESTABLISHED,RELATED -j ACCEPT</xsl:text>
			<xsl:text>&#xa;iptables -t filter -A OUTPUT -m state --state ESTABLISHED,RELATED -j ACCEPT</xsl:text>
			<xsl:text>&#xa;iptables -t filter -A FORWARD -m state --state ESTABLISHED,RELATED -j ACCEPT</xsl:text>
		</xsl:if>
		
		<xsl:text>&#xa;echo '</xsl:text><xsl:value-of select="@conntrack_max"/><xsl:text>' > /proc/sys/net/ipv4/ip_conntrack_max</xsl:text>
		
	</xsl:template>
	
	<xsl:template match="POLICY/IDS">
		<xsl:if test="@catch_orpheans='yes'">
			<xsl:if test="@log='yes'">
				<xsl:text>&#xa;iptables -t filter -A INPUT -f -m state --state NEW </xsl:text>
				<xsl:text>-j LOG --log-prefix "ORPHEAN_PACKET : "</xsl:text>
				<xsl:text>&#xa;iptables -t filter -A OUTPUT -f -m state --state NEW </xsl:text>
				<xsl:text>-j LOG --log-prefix "ORPHEAN_PACKET : "</xsl:text>
				<xsl:text>&#xa;iptables -t filter -A FORWARD -f -m state --state NEW </xsl:text>
				<xsl:text>-j LOG --log-prefix "ORPHEAN_PACKET : "</xsl:text>
			</xsl:if>
			<xsl:text>&#xa;iptables -t filter -A INPUT -f -j DROP</xsl:text>
			<xsl:text>&#xa;iptables -t filter -A OUTPUT -f -j DROP</xsl:text>
			<xsl:text>&#xa;iptables -t filter -A FORWARD -f -j DROP</xsl:text>
		</xsl:if>
	
		<xsl:if test="@catch_synscan='yes'">
			<xsl:if test="@log='yes'">
				<xsl:text>&#xa;iptables -t filter -A INPUT -p TCP -m state --state NEW ! --syn -j LOG --log-prefix "SYNSCAN_PACKET : "</xsl:text>
				<xsl:text>&#xa;iptables -t filter -A OUTPUT -p TCP -m state --state NEW ! --syn -j LOG --log-prefix "SYNSCAN_PACKET : "</xsl:text>
				<xsl:text>&#xa;iptables -t filter -A FORWARD -p TCP -m state --state NEW ! --syn -j LOG --log-prefix "SYNSCAN_PACKET : "</xsl:text>
			</xsl:if>
			<xsl:text>&#xa;iptables -t filter -A INPUT -p TCP -m state --state NEW ! --syn -j DROP</xsl:text>
			<xsl:text>&#xa;iptables -t filter -A OUTPUT -p TCP -m state --state NEW ! --syn -j DROP</xsl:text>
			<xsl:text>&#xa;iptables -t filter -A FORWARD -p TCP -m state --state NEW ! --syn -j DROP</xsl:text>
		</xsl:if>
		
		<xsl:if test="@catch_invalid='yes'">
			<xsl:if test="@log='yes'">
				<xsl:text>&#xa;iptables -t filter -A INPUT -m state --state INVALID -j LOG --log-prefix "INVALID_PACKET : "</xsl:text>
				<xsl:text>&#xa;iptables -t filter -A OUTPUT -m state --state INVALID -j LOG --log-prefix "INVALID_PACKET : "</xsl:text>
				<xsl:text>&#xa;iptables -t filter -A FORWARD -m state --state INVALID -j LOG --log-prefix "INVALID_PACKET : "</xsl:text>
			</xsl:if>
			<xsl:text>&#xa;iptables -t filter -A INPUT -m state --state INVALID -j DROP</xsl:text>
			<xsl:text>&#xa;iptables -t filter -A OUTPUT -m state --state INVALID -j DROP</xsl:text>
			<xsl:text>&#xa;iptables -t filter -A FORWARD -m state --state INVALID -j DROP</xsl:text>
		</xsl:if>
		
		<xsl:if test="@log_martians='yes'">
			<xsl:text>&#xa;# Log any packets that have IP addresses that should not exist</xsl:text>
			<xsl:text>&#xa;enable /proc/sys/net/ipv4/conf/all/log_martians</xsl:text>
		</xsl:if>
		<xsl:if test="@log_martians='no'">
			<xsl:text>&#xa;disable /proc/sys/net/ipv4/conf/all/log_martians</xsl:text>
		</xsl:if>
	</xsl:template>
	
	<xsl:template match="COMMENT">
		<xsl:text>&#xa;# </xsl:text><xsl:value-of select="@text" />
	</xsl:template>
	
	<xsl:template match="FW_HOST">
		<xsl:for-each select="INPUT">
			<xsl:for-each select="COMMENT">
				<xsl:text>&#xa;# </xsl:text><xsl:value-of select="@text" />
			</xsl:for-each>
			
			<xsl:for-each select="LOG_RULE">
				<xsl:text>&#xa;iptables -t filter -A INPUT</xsl:text>
				<xsl:if test="ancestor::*/@in">
					<xsl:text> -i </xsl:text><xsl:value-of select="ancestor::*/@in"/> 
				</xsl:if>
				<xsl:if test="@limit">
					<xsl:text> -m limit --limit </xsl:text><xsl:value-of select="@limit"/>
					<xsl:if test="@burst">
						<xsl:text> --limit-burst </xsl:text><xsl:value-of select="@burst"/>
					</xsl:if>
				</xsl:if>
				<xsl:if test="@saddress">
					<xsl:text> -s </xsl:text><xsl:value-of select="@saddress"/> 
				</xsl:if>
				<xsl:if test="@daddress">
					<xsl:text> -d </xsl:text><xsl:value-of select="@daddress"/> 
				</xsl:if>
				<xsl:if test="@proto">
					<xsl:text> -p </xsl:text><xsl:value-of select="@proto"/> 
				</xsl:if>
				<xsl:if test="@tcp_flag">
					<xsl:text> --</xsl:text><xsl:value-of select="@tcp_flag"/>
				</xsl:if>
				<xsl:if test="@proto='icmp'">
					<xsl:if test="@icmp_type">
						<xsl:text> --icmp-type </xsl:text><xsl:value-of select="@icmp_type"/> 
					</xsl:if>	
				</xsl:if>
				<xsl:if test="@dport">
					<xsl:text> --dport </xsl:text><xsl:value-of select="@dport"/>
				</xsl:if>
				<xsl:if test="@sport">
					<xsl:text> --sport </xsl:text><xsl:value-of select="@sport"/>
				</xsl:if>
				<xsl:if test="@tos">
					<xsl:text> -m tos --tos </xsl:text><xsl:value-of select="@tos"/>
				</xsl:if>
				<xsl:if test="@ttl">
					<xsl:text> -m ttl --ttl </xsl:text><xsl:value-of select="@ttl"/>
				</xsl:if>			
				<xsl:text> -j LOG</xsl:text>
				<xsl:if test="@prefix">
					<xsl:text> --log-prefix "</xsl:text><xsl:value-of select="@prefix"/><xsl:text>"</xsl:text>
				</xsl:if>
				<xsl:text> # </xsl:text><xsl:value-of select="." />
			</xsl:for-each>
			
			<xsl:for-each select="FILTER_RULE">
				<xsl:text>&#xa;iptables -t filter -A INPUT</xsl:text>
				<xsl:if test="@saddress">
					<xsl:text> -s </xsl:text><xsl:value-of select="@daddress"/> 
				</xsl:if>
				<xsl:if test="@daddress">
					<xsl:text> -d </xsl:text><xsl:value-of select="@daddress"/> 
				</xsl:if>
				<xsl:if test="ancestor::*/@in">
						<xsl:text> -i </xsl:text><xsl:value-of select="ancestor::*/@in"/> 
					</xsl:if>
				<xsl:if test="@limit">
					<xsl:text> -m limit --limit </xsl:text><xsl:value-of select="@limit"/>
					<xsl:if test="@burst">
						<xsl:text> --limit-burst </xsl:text><xsl:value-of select="@burst"/>
					</xsl:if>
				</xsl:if>
				<xsl:if test="@proto">
					<xsl:text> -p </xsl:text><xsl:value-of select="@proto"/> 
				</xsl:if>
				<xsl:if test="@tcp_flag">
					<xsl:text> --</xsl:text><xsl:value-of select="@tcp_flag"/>
				</xsl:if>
				<xsl:if test="@proto='icmp'">
					<xsl:if test="@icmp_type">
						<xsl:text> --icmp-type </xsl:text><xsl:value-of select="@icmp_type"/> 
					</xsl:if>	
				</xsl:if>
				<xsl:if test="@dport">
					<xsl:text> --dport </xsl:text><xsl:value-of select="@dport"/>
				</xsl:if>
				<xsl:if test="@sport">
					<xsl:text> --sport </xsl:text><xsl:value-of select="@sport"/>
				</xsl:if>
				<xsl:if test="@tos">
					<xsl:text> -m tos --tos </xsl:text><xsl:value-of select="@tos"/>
				</xsl:if>
				<xsl:if test="@ttl">
					<xsl:text> -m ttl --ttl </xsl:text><xsl:value-of select="@ttl"/>
				</xsl:if>			
				<xsl:text> -j </xsl:text><xsl:value-of select="@action"/>
				<xsl:text> # </xsl:text><xsl:value-of select="." />
			</xsl:for-each>
			
			<xsl:for-each select="TOS_RULE">
				<xsl:text>&#xa;iptables -t mangle -A PREROUTING</xsl:text>
				<xsl:if test="ancestor::*/@in">
					<xsl:text> -i </xsl:text><xsl:value-of select="ancestor::*/@in"/> 
				</xsl:if>
				<xsl:if test="@saddress">
					<xsl:text> -s </xsl:text><xsl:value-of select="@daddress"/> 
				</xsl:if>
				<xsl:if test="@daddress">
					<xsl:text> -d </xsl:text><xsl:value-of select="@daddress"/> 
				</xsl:if>
				<xsl:if test="@proto">
					<xsl:text> -p </xsl:text><xsl:value-of select="@proto"/> 
				</xsl:if>
				<xsl:if test="@dport">
					<xsl:text> --dport </xsl:text><xsl:value-of select="@dport"/>
				</xsl:if>
				<xsl:if test="@sport">
					<xsl:text> --sport </xsl:text><xsl:value-of select="@sport"/>
				</xsl:if>		
				<xsl:if test="@tos">
					<xsl:text> -m tos --tos </xsl:text><xsl:value-of select="@tos"/>
				</xsl:if>
				<xsl:if test="@ttl">
					<xsl:text> -m ttl --ttl </xsl:text><xsl:value-of select="@ttl"/>
				</xsl:if>
				<xsl:text> -j TOS</xsl:text>
				<xsl:text> --set-tos </xsl:text><xsl:value-of select="@set_tos"/>
				<xsl:text> # </xsl:text><xsl:value-of select="." />
			</xsl:for-each>
		 </xsl:for-each>
		 
		 <xsl:for-each select="OUTPUT">
			<xsl:for-each select="COMMENT">
				<xsl:text>&#xa;# </xsl:text><xsl:value-of select="@text" />
			</xsl:for-each>
			
			<xsl:for-each select="LOG_RULE">
				<xsl:text>&#xa;iptables -t filter -A OUTPUT</xsl:text>
				<xsl:if test="ancestor::*/@out">
					<xsl:text> -o </xsl:text><xsl:value-of select="ancestor::*/@out"/> 
				</xsl:if>
				<xsl:if test="@limit">
					<xsl:text> -m limit --limit </xsl:text><xsl:value-of select="@limit"/>
					<xsl:if test="@burst">
						<xsl:text> --limit-burst </xsl:text><xsl:value-of select="@burst"/>
					</xsl:if>
				</xsl:if>
				<xsl:if test="@saddress">
					<xsl:text> -s </xsl:text><xsl:value-of select="@saddress"/> 
				</xsl:if>
				<xsl:if test="@daddress">
					<xsl:text> -d </xsl:text><xsl:value-of select="@daddress"/> 
				</xsl:if>
				<xsl:if test="@proto">
					<xsl:text> -p </xsl:text><xsl:value-of select="@proto"/> 
				</xsl:if>
				<xsl:if test="@tcp_flag">
					<xsl:text> --</xsl:text><xsl:value-of select="@tcp_flag"/>
				</xsl:if>
				<xsl:if test="@proto='icmp'">
					<xsl:if test="@icmp_type">
						<xsl:text> --icmp-type </xsl:text><xsl:value-of select="@icmp_type"/> 
					</xsl:if>	
				</xsl:if>
				<xsl:if test="@dport">
					<xsl:text> --dport </xsl:text><xsl:value-of select="@dport"/>
				</xsl:if>
				<xsl:if test="@sport">
					<xsl:text> --sport </xsl:text><xsl:value-of select="@sport"/>
				</xsl:if>
				<xsl:if test="@tos">
					<xsl:text> -m tos --tos </xsl:text><xsl:value-of select="@tos"/>
				</xsl:if>
				<xsl:if test="@ttl">
					<xsl:text> -m ttl --ttl </xsl:text><xsl:value-of select="@ttl"/>
				</xsl:if>			
				<xsl:text> -j LOG</xsl:text>
				<xsl:if test="@prefix">
					<xsl:text> --log-prefix "</xsl:text><xsl:value-of select="@prefix"/><xsl:text>"</xsl:text>
				</xsl:if>
				<xsl:text> # </xsl:text><xsl:value-of select="." />
			</xsl:for-each>
			
			<xsl:for-each select="FILTER_RULE">
				<xsl:text>&#xa;iptables -t filter -A OUTPUT</xsl:text>
				<xsl:if test="@saddress">
					<xsl:text> -s </xsl:text><xsl:value-of select="@daddress"/> 
				</xsl:if>
				<xsl:if test="@daddress">
					<xsl:text> -d </xsl:text><xsl:value-of select="@daddress"/> 
				</xsl:if>
				<xsl:if test="ancestor::*/@out">
						<xsl:text> -o </xsl:text><xsl:value-of select="ancestor::*/@out"/> 
					</xsl:if>
				<xsl:if test="@limit">
					<xsl:text> -m limit --limit </xsl:text><xsl:value-of select="@limit"/>
					<xsl:if test="@burst">
						<xsl:text> --limit-burst </xsl:text><xsl:value-of select="@burst"/>
					</xsl:if>
				</xsl:if>
				<xsl:if test="@proto">
					<xsl:text> -p </xsl:text><xsl:value-of select="@proto"/> 
				</xsl:if>
				<xsl:if test="@tcp_flag">
					<xsl:text> --</xsl:text><xsl:value-of select="@tcp_flag"/>
				</xsl:if>
				<xsl:if test="@proto='icmp'">
					<xsl:if test="@icmp_type">
						<xsl:text> --icmp-type </xsl:text><xsl:value-of select="@icmp_type"/> 
					</xsl:if>	
				</xsl:if>
				<xsl:if test="@dport">
					<xsl:text> --dport </xsl:text><xsl:value-of select="@dport"/>
				</xsl:if>
				<xsl:if test="@sport">
					<xsl:text> --sport </xsl:text><xsl:value-of select="@sport"/>
				</xsl:if>
				<xsl:if test="@tos">
					<xsl:text> -m tos --tos </xsl:text><xsl:value-of select="@tos"/>
				</xsl:if>
				<xsl:if test="@ttl">
					<xsl:text> -m ttl --ttl </xsl:text><xsl:value-of select="@ttl"/>
				</xsl:if>			
				<xsl:if test="ancestor::*/@process">
					<xsl:text> -m owner --cmd-owner </xsl:text><xsl:value-of select="ancestor::*/@process"/> 
				</xsl:if>
				<xsl:if test="ancestor::*/@group">
					<xsl:text> -m owner --gid-owner </xsl:text><xsl:value-of select="ancestor::*/@group"/> 
				</xsl:if>
				<xsl:if test="ancestor::*/@user">
					<xsl:text> -m owner --uid-owner </xsl:text><xsl:value-of select="ancestor::*/@user"/> 
				</xsl:if>
				<xsl:text> -j </xsl:text><xsl:value-of select="@action"/>
				<xsl:text> # </xsl:text><xsl:value-of select="." />
			</xsl:for-each>
			
			<xsl:for-each select="TOS_RULE">
				<xsl:text>&#xa;iptables -t mangle -A POSTROUTING</xsl:text>
				<xsl:if test="ancestor::*/@out">
					<xsl:text> -o </xsl:text><xsl:value-of select="ancestor::*/@out"/> 
				</xsl:if>
				<xsl:if test="@saddress">
					<xsl:text> -s </xsl:text><xsl:value-of select="@daddress"/> 
				</xsl:if>
				<xsl:if test="@daddress">
					<xsl:text> -d </xsl:text><xsl:value-of select="@daddress"/> 
				</xsl:if>
				<xsl:if test="@proto">
					<xsl:text> -p </xsl:text><xsl:value-of select="@proto"/> 
				</xsl:if>
				<xsl:if test="@dport">
					<xsl:text> --dport </xsl:text><xsl:value-of select="@dport"/>
				</xsl:if>
				<xsl:if test="@sport">
					<xsl:text> --sport </xsl:text><xsl:value-of select="@sport"/>
				</xsl:if>		
				<xsl:if test="@tos">
					<xsl:text> -m tos --tos </xsl:text><xsl:value-of select="@tos"/>
				</xsl:if>
				<xsl:if test="@ttl">
					<xsl:text> -m ttl --ttl </xsl:text><xsl:value-of select="@ttl"/>
				</xsl:if>
				<xsl:text> -j TOS</xsl:text>
				<xsl:text> --set-tos </xsl:text><xsl:value-of select="@set_tos"/>
				<xsl:text> # </xsl:text><xsl:value-of select="." />
			</xsl:for-each>
		 </xsl:for-each>
		 
	</xsl:template>
	
	<xsl:template match="GATEWAY">
		<xsl:for-each select="FORWARD">
			<xsl:for-each select="COMMENT">
				<xsl:text>&#xa;# </xsl:text><xsl:value-of select="@text" />
			</xsl:for-each>
			<xsl:text>&#xa;iptables -t filter -N FORWARD-</xsl:text>
			<xsl:value-of select="@in"/>
			<xsl:text>-</xsl:text>
			<xsl:value-of select="@out"/>	
			<xsl:text>&#xa;iptables -t filter -A FORWARD</xsl:text>
			<xsl:if test="@in">
				<xsl:text> -i </xsl:text><xsl:value-of select="@in"/> 
			</xsl:if>
			<xsl:if test="@out">
				<xsl:text> -o </xsl:text><xsl:value-of select="@out"/> 
			</xsl:if>
			<xsl:text> -j FORWARD-</xsl:text><xsl:value-of select="@in"/><xsl:text>-</xsl:text><xsl:value-of select="@out"/>	
					
			<xsl:for-each select="LOG_RULE">
				<xsl:text>&#xa;iptables -t filter -A FORWARD-</xsl:text>
				<xsl:value-of select="ancestor::*/@in"/>
				<xsl:text>-</xsl:text>
				<xsl:value-of select="ancestor::*/@out"/>
				<xsl:if test="@limit">
					<xsl:text> -m limit --limit </xsl:text><xsl:value-of select="@limit"/>
					<xsl:if test="@burst">
						<xsl:text> --limit-burst </xsl:text><xsl:value-of select="@burst"/>
					</xsl:if>
				</xsl:if>
				<xsl:if test="@saddress">
					<xsl:text> -s </xsl:text><xsl:value-of select="@saddress"/> 
				</xsl:if>
				<xsl:if test="@daddress">
					<xsl:text> -d </xsl:text><xsl:value-of select="@daddress"/> 
				</xsl:if>
				<xsl:if test="@proto">
					<xsl:text> -p </xsl:text><xsl:value-of select="@proto"/> 
				</xsl:if>
				<xsl:if test="@tcp_flag">
					<xsl:text> --</xsl:text><xsl:value-of select="@tcp_flag"/>
				</xsl:if>
				<xsl:if test="@proto='icmp'">
					<xsl:if test="@icmp_type">
						<xsl:text> --icmp-type </xsl:text><xsl:value-of select="@icmp_type"/> 
					</xsl:if>	
				</xsl:if>
				<xsl:if test="@dport">
					<xsl:text> --dport </xsl:text><xsl:value-of select="@dport"/>
				</xsl:if>
				<xsl:if test="@sport">
					<xsl:text> --sport </xsl:text><xsl:value-of select="@sport"/>
				</xsl:if>
				<xsl:if test="@tos">
					<xsl:text> -m tos --tos </xsl:text><xsl:value-of select="@tos"/>
				</xsl:if>
				<xsl:if test="@ttl">
					<xsl:text> -m ttl --ttl </xsl:text><xsl:value-of select="@ttl"/>
				</xsl:if>			
				<xsl:text> -j LOG</xsl:text>
				<xsl:if test="@prefix">
					<xsl:text> --log-prefix "</xsl:text><xsl:value-of select="@prefix"/><xsl:text>"</xsl:text>
				</xsl:if>
				<xsl:text> # </xsl:text><xsl:value-of select="." />
			</xsl:for-each>
			
			<xsl:for-each select="FILTER_RULE">
				<xsl:text>&#xa;iptables -t filter -A FORWARD-</xsl:text>
				<xsl:value-of select="ancestor::*/@in"/>
				<xsl:text>-</xsl:text>
				<xsl:value-of select="ancestor::*/@out"/>
				<xsl:if test="@saddress">
					<xsl:text> -s </xsl:text><xsl:value-of select="@daddress"/> 
				</xsl:if>
				<xsl:if test="@daddress">
					<xsl:text> -d </xsl:text><xsl:value-of select="@daddress"/> 
				</xsl:if>
				<xsl:if test="@limit">
					<xsl:text> -m limit --limit </xsl:text><xsl:value-of select="@limit"/>
					<xsl:if test="@burst">
						<xsl:text> --limit-burst </xsl:text><xsl:value-of select="@burst"/>
					</xsl:if>
				</xsl:if>
				<xsl:if test="@proto">
					<xsl:text> -p </xsl:text><xsl:value-of select="@proto"/> 
				</xsl:if>
				<xsl:if test="@tcp_flag">
					<xsl:text> --</xsl:text><xsl:value-of select="@tcp_flag"/>
				</xsl:if>
				<xsl:if test="@proto='icmp'">
					<xsl:if test="@icmp_type">
						<xsl:text> --icmp-type </xsl:text><xsl:value-of select="@icmp_type"/> 
					</xsl:if>	
				</xsl:if>
				<xsl:if test="@dport">
					<xsl:text> --dport </xsl:text><xsl:value-of select="@dport"/>
				</xsl:if>
				<xsl:if test="@sport">
					<xsl:text> --sport </xsl:text><xsl:value-of select="@sport"/>
				</xsl:if>		
				<xsl:if test="@tos">
					<xsl:text> -m tos --tos </xsl:text><xsl:value-of select="@tos"/>
				</xsl:if>
				<xsl:if test="@ttl">
					<xsl:text> -m ttl --ttl </xsl:text><xsl:value-of select="@ttl"/>
				</xsl:if>
				<xsl:text> -j </xsl:text><xsl:value-of select="@action"/>
				<xsl:text> # </xsl:text><xsl:value-of select="." />
			</xsl:for-each>
			
			<xsl:for-each select="NAT_RULE">
				<xsl:if test="@action='DNAT'">
					<xsl:text>&#xa;iptables -t nat -A PREROUTING </xsl:text>
				</xsl:if>
				<xsl:if test="@action='SNAT'">
					<xsl:text>&#xa;iptables -t nat -A POSTROUTING </xsl:text>
				</xsl:if>
				<xsl:if test="@action='MASQUERADE'">
					<xsl:text>&#xa;iptables -t nat -A POSTROUTING </xsl:text> 
				</xsl:if>
				<xsl:if test="@saddress">
					<xsl:text> -s </xsl:text><xsl:value-of select="@saddress"/> 
				</xsl:if>
				<xsl:if test="@daddress">
					<xsl:text> -d </xsl:text><xsl:value-of select="@daddress"/> 
				</xsl:if>
				<xsl:if test="@action='DNAT'">
					<xsl:if test="ancestor::*/@in">
						<xsl:text> -i </xsl:text><xsl:value-of select="ancestor::*/@in"/> 
					</xsl:if> 
				</xsl:if>
				<xsl:if test="@action='SNAT'">
					<xsl:if test="ancestor::*/@out">
						<xsl:text> -o </xsl:text><xsl:value-of select="ancestor::*/@out"/> 
					</xsl:if> 
				</xsl:if>
				<xsl:if test="@action='MASQUERADE'">
					<xsl:if test="ancestor::*/@out">
						<xsl:text> -o </xsl:text><xsl:value-of select="ancestor::*/@out"/> 
					</xsl:if> 
				</xsl:if>
				<xsl:if test="@proto">
					<xsl:text> -p </xsl:text><xsl:value-of select="@proto"/> 
				</xsl:if>
				<xsl:if test="@tcp_flag">
					<xsl:text> --</xsl:text><xsl:value-of select="@tcp_flag"/>
				</xsl:if>
				<xsl:if test="@proto='icmp'">
					<xsl:if test="@icmp_type">
						<xsl:text> --icmp-type </xsl:text><xsl:value-of select="@icmp_type"/> 
					</xsl:if>	
				</xsl:if>
				<xsl:if test="@dport">
					<xsl:text> --dport </xsl:text><xsl:value-of select="@dport"/>
				</xsl:if>
				<xsl:if test="@sport">
					<xsl:text> --sport </xsl:text><xsl:value-of select="@sport"/>
				</xsl:if>
				<xsl:text> -j </xsl:text><xsl:value-of select="@action"/>		
				<xsl:if test="@nat_to">
					<xsl:if test="@action='DNAT'">
						<xsl:text> --to-destination </xsl:text><xsl:value-of select="@nat_to"/>
					</xsl:if>
					<xsl:if test="@action='SNAT'">
						<xsl:text> --to-source </xsl:text><xsl:value-of select="@nat_to"/>
					</xsl:if>
				</xsl:if>
				<xsl:text> # </xsl:text><xsl:value-of select="." />
			</xsl:for-each>
			
			<xsl:for-each select="TOS_RULE">
				<xsl:text>&#xa;iptables -t mangle -A FORWARD</xsl:text>
				<xsl:if test="ancestor::*/@in">
					<xsl:text> -i </xsl:text><xsl:value-of select="ancestor::*/@in"/> 
				</xsl:if>
				<xsl:if test="ancestor::*/@out">
					<xsl:text> -o </xsl:text><xsl:value-of select="ancestor::*/@out"/> 
				</xsl:if>
				<xsl:if test="@saddress">
					<xsl:text> -s </xsl:text><xsl:value-of select="@daddress"/> 
				</xsl:if>
				<xsl:if test="@daddress">
					<xsl:text> -d </xsl:text><xsl:value-of select="@daddress"/> 
				</xsl:if>
				<xsl:if test="@proto">
					<xsl:text> -p </xsl:text><xsl:value-of select="@proto"/> 
				</xsl:if>
				<xsl:if test="@dport">
					<xsl:text> --dport </xsl:text><xsl:value-of select="@dport"/>
				</xsl:if>
				<xsl:if test="@sport">
					<xsl:text> --sport </xsl:text><xsl:value-of select="@sport"/>
				</xsl:if>		
				<xsl:if test="@tos">
					<xsl:text> -m tos --tos </xsl:text><xsl:value-of select="@tos"/>
				</xsl:if>
				<xsl:if test="@ttl">
					<xsl:text> -m ttl --ttl </xsl:text><xsl:value-of select="@ttl"/>
				</xsl:if>
				<xsl:text> -j TOS</xsl:text>
				<xsl:text> --set-tos </xsl:text><xsl:value-of select="@set_tos"/>
				<xsl:text> # </xsl:text><xsl:value-of select="." />
			</xsl:for-each>
			
			
		 </xsl:for-each>
	</xsl:template>

</xsl:stylesheet>
EOT
	my $stylesheet = $xslt->parse_stylesheet($sheet);
	my $transformed = $stylesheet->transform($xml);
	my $output_string = $stylesheet->output_string($transformed);
	
	#------------------------------------------------------------ 
	
	system ($output_string);
	
	open( SCRIPT, ">./netfilter.sh" ) || die "Error : can't open file netfilter.sh !\n";
	print SCRIPT $output_string;
	my $chmod="chmod 711 ./netfilter.sh";
	system ($chmod);
	close(XSL);
	close(XML);
	close(SCRIPT);
}else{
	&usage();
}

sub usage {
	print ("\n\nusage :  xmlfilter.pl [-c Netfilter config file ]\n");
	die("goodbye :))");
}
